


#include "h/Graphique.h"

#include "h/SPI.h"
#include "h/Platine.h"
#include "h/Utilitaires.h"
#include "h/Font5x7.h"	/* Police de caractres de type Arial 5 X 7 points. */



unsigned char g_EtatDuRetroeclairage = 0;

unsigned char g_EtatLEDGraphique = 0;

unsigned char g_BlocActuel = 0;



#define x_max 128	/* Abscisse de la dernire colonne en pixels. */
#define y_max 64	/* Ordonne de la dernire ligne en pixels.	*/
#define YLigne_max 8	/* Hauteur d'une page (8 pixels). */
#define NOMBRE_OCTETS_ECRAN 1024			  /* 1024 octets reprsentent la totalit de l'cran. */



#define LCD_E 0x08
#define LCD_RS 0x20
#define LCD_RW 0x10

#define LCD_CS1 0x04
#define LCD_CS2 0x02
#define LCD_RST 0x01



struct adresse	/* Structure contenant les positions du curseur de l'afficheur graphique */
{
	unsigned char X;	/* Abscisse du curseur */
	unsigned char YLigne;	/* Ordonne du curseur, qui correspond en fait  la ligne sur laquelle il se situe */
}Adresse = {1, 1};	/* Les coordonnes du curseur (X, YLigne) prennent les valeurs initiales (1, 1). */



/* Constante utile pour la conversion HEXADECIMAL / ASCII.
Elle contient les codes ASCII des caractres de 0  F.
Exemple : h[12] = Code ASCII (0x43) du caractre  C . */



const unsigned char Hexadecimaux[16] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'}; 



/**************************************************************************************
void EnvoyerControleGraphique(unsigned char Controle)

Envoie l'octet  Controle  de contrle pour modifier la configuration de l'afficheur.

**************************************************************************************/
void EnvoyerControleGraphique(unsigned char Controle)
{
	unsigned char Etat = 0x00;

	Etat |= (g_EtatLEDGraphique | g_EtatDuRetroeclairage | g_BlocActuel);
	
	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, Controle, Etat);	/*  Controle  est prsent
																			aux broches  DB0    DB7  
																			de l'afficheur alphanumrique,
																			et les broches  RW  ,  RS  et  E  sont
																			mises   0 .
																			Le fait que la broche  RS  soit mise   0  indique
																			 l'afficheur graphique qu'il va recevoir un octet de commandes,
																			et non pas un octet de donnes. */

/* LCD_RW = 0;
LCD_RS = 0;
LCD_E = 0; */

	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, Controle, (Etat | LCD_E));	/*  Controle  est prsent
																					aux broches  DB0    DB7  
																					de l'afficheur alphanumrique,
																					et la broche  E  est
																					mise   1 . */

/* LCD_RW = 0;
LCD_RS = 0;
LCD_E = 1; */

	Delai(3);
		
	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, Controle, Etat);	/*  Controle  est prsent
																			aux broches  DB0    DB7  
																			de l'afficheur alphanumrique,
																			et la broche  E  est
																			remise   0 , ce qui valide l'envoi de l'octet  Tampon . */

/* LCD_RW = 0;
LCD_RS = 0;
LCD_E = 0; */

	Delai(2);
}


/**************************************************************************************
void EnvoyerDonneeGraphique(unsigned char Donnee)

Envoie l'octet  Donnee  de donnes pour modifier l'affichage visible.

**************************************************************************************/
void EnvoyerDonneeGraphique(unsigned char Donnee)
{
	unsigned char Etat = 0x00;

	Etat |= (g_EtatLEDGraphique | g_EtatDuRetroeclairage | g_BlocActuel);

	if(Adresse.YLigne <= 8)
	{
		if(Adresse.X == 64)
		{
			PositionnerCurseurXYGraphique(65, Adresse.YLigne);

		}
		if(Adresse.X > 128)
		{
			Adresse.YLigne++;
			PositionnerCurseurXYGraphique(1, Adresse.YLigne);
			Adresse.X = 0;
		}
	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, Donnee, (Etat | LCD_RS));	/*  Donnee  est prsente
																					aux broches  DB0    DB7  
																					de l'afficheur alphanumrique,
																					les broches  RW  et  E  sont
																					mises   0 , alors que la broche   RS  est mise   1 .
																					Le fait que la broche  RS  soit mise   1  indique
																					 l'afficheur graphique qu'il va recevoir un octet de donnes,
																					et non pas un octet de commandes. */

/* LCD_RW = 0; */
/* LCD_RS = 1; */
/* LCD_E = 0; */

	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, Donnee, (Etat | (LCD_RS | LCD_E)));	/*  Donnee  est prsente
																					aux broches  DB0    DB7  
																					de l'afficheur alphanumrique,
																					et la broche  E  est
																					mise   1 . */

/* LCD_RW = 0; */
/* LCD_RS = 1; */
/* LCD_E = 1; */

	Delai(3);

	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, Donnee, (Etat | LCD_RS));	/*  Donnee  est prsente
																					aux broches  DB0    DB7  
																					de l'afficheur alphanumrique,
																					et la broche  E  est
																					remise   0 , ce qui valide l'envoi de l'octet  Donnee . */

/* LCD_RW = 0; */
/* LCD_RS = 1; */
/* LCD_E = 0; */

	Delai(2);

	Adresse.X++;
	}
}

/**************************************************************************************
unsigned char LireDonneeGraphique(void)

Retourne la valeur de l'octet affich  la position du curseur.

**************************************************************************************/
unsigned char LireDonneeGraphique(void)
{
	unsigned char Etat = 0x00;

	unsigned char Tampon1 = 0x00;
	unsigned char Tampon2 = 0x00;

	Etat |= (g_EtatLEDGraphique | g_EtatDuRetroeclairage | g_BlocActuel);

	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x00, 0xFF, 0x00);	/* DDR_Donnee = 0x00; */                            	/*  PORT_Donnee  en entre. */

/* LCD_RS = 1; */                               	/* Envoie un octet de donne. */
/* LCD_RW = 1; */                               	/* Lecture de la RAM du LCD. */
/* LCD_E = 1; */
   
	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, 0x00, (Etat | (LCD_RS | LCD_RW | LCD_E)));	/* Les broches  RS  ,  RW  et  E   
																				sont mises   1 . */

	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, 0x00, (Etat | LCD_RS));	/* La broche  RS 
																				est mise   1 . */

	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, 0x00, (Etat | (LCD_RS | LCD_RW | LCD_E)));	/* Les broches  RS  ,  RW  et  E   
																				sont mises   1 . */

	LireRegistreExpanseur(AdresseModuleGraphique, 0x13, &Tampon1, &Tampon2);	/* Tampon2 = PORT_Donnee; */                         	/* L'octet lu est mis dans  Tampon2 . */

/* LCD_E = 0; */                                	/* Validation par l'impulsion E. */
/* LCD_RW = 0; */                               	/* Ecriture dans la RAM du LCD. */

	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, 0x00, (Etat | LCD_RS));	/* Les broches  E  et  RW  
																				sont mises   0 .
																				La broche  RS  est encore mise   1 . */

	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x00, 0x00, 0x00);	/* DDR_Donnee = 0xFF; */                          	/*  PORT_Donnee  en sortie. */

	return Tampon2;                              	/* Retourne la valeur de l'octet lu. */
}

/**************************************************************************************
void SelectionnerBlocGraphique(unsigned char Numero)

Slectionne le ct gauche (64 X 64 pixels) , ou le ct droit (64 X 64 pixels) du LCD.

Numero = 1 : Ct droit ; Numero = 0 : Ct gauche.

**************************************************************************************/
void SelectionnerBlocGraphique(unsigned char Numero)
{
	unsigned char Etat = 0x00;

	Etat |= (g_EtatLEDGraphique | g_EtatDuRetroeclairage);

	switch(Numero)
	{
		case 0 :
		{
			EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, 0x00, (Etat | (LCD_CS1 | LCD_RST)));	/* La broche  CS1  de l'afficheur alphanumrique
																						est mise   1 ,
																						et la broche  CS2 
																						est mise   0 ,
																						ce qui permet de slectionner le bloc 0. */

/* LCD_CS1 = 1; */
/* LCD_CS2 = 0; */

			g_BlocActuel = (LCD_CS1 | LCD_RST);

			break;
		}
		case 1 :
		{
			EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, 0x00, (Etat | (LCD_CS2 | LCD_RST)));	/* La broche  CS1  de l'afficheur alphanumrique
																						est mise   0 ,
																						et la broche  CS2 
																						est mise   1 ,
																						ce qui permet de slectionner le bloc 1. */

/* LCD_CS1 = 0; */
/* LCD_CS2 = 1; */

			g_BlocActuel = (LCD_CS2 | LCD_RST);

			break;
		}
		default:
		{
			break;
		}
	}
}

/**************************************************************************************
void InitialisationAfficheurGraphique(unsigned char Positif)

Initialise l'afficheur LCD et les PORTS utiliss.

Positif=0: cran  fond blanc; Positif=1: cran  fond noir.

**************************************************************************************/
void InitialiserAfficheurGraphique(unsigned char Positif)
{
	unsigned char Etat = 0x00;

	Etat |= (g_EtatLEDGraphique | g_EtatDuRetroeclairage);
	
	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, 0x00, Etat);	/* Les broches  E ,  RW  et  RST  de l'afficheur alphanumrique
																							sont mises   0  */

	EcrireRegistreExpanseur(AdresseModuleGraphique, 0x12, 0x00, (Etat | LCD_RST));	/* Les broches  E  et  RW  de l'afficheur alphanumrique
																				sont mises   0 ,
																				et la broche  RST  est mise   1  */

	SelectionnerBlocGraphique(0);
	EnvoyerControleGraphique(0xC0);	/* RAM 0 sur le haut de l'afficheur ct gauche. */
	EnvoyerControleGraphique(0x3F);	/* LCD allum. */
	SelectionnerBlocGraphique(1);
	EnvoyerControleGraphique(0xC0);	/* RAM 0 sur le haut de l'afficheur ct droit. */
	EnvoyerControleGraphique(0x3F);	/* LCD allum. */
	EffacerAfficheurGraphique(Positif);	/* Efface l'cran et positionne le curseur au point (1, 1). */
}

/**************************************************************************************
void PositionnerCurseurXYGraphique(unsigned char X1, unsigned char YLigne1)

Positionne le curseur aux coordonnes (x1, YLigne1) : 1 <= X1 <= 128,  
et 1 <= YLigne1 <= 8  (pages).

**************************************************************************************/
void PositionnerCurseurXYGraphique(unsigned char X1, unsigned char YLigne1)
{
	if(X1 <= 64)	/* Si l'on veut positionner le curseur dans la premire moiti de l'afficheur graphique */
	{
		SelectionnerBlocGraphique(0);	/* Ct gauche. */
		Delai(10);
		EnvoyerControleGraphique(0x40 + X1 - 1);	/* Colonne (X1 - 1). */
		EnvoyerControleGraphique(0xB8 + YLigne1 - 1);	/* incrmente page. */
	}
	else	/* Si l'on veut positionner le curseur dans la deuxime moiti de l'afficheur graphique */
	{
		SelectionnerBlocGraphique(1);	/* ct droit. */
		EnvoyerControleGraphique(0x40 + X1 - 65);	/* colonne (X1 - 1 - 64). */
		EnvoyerControleGraphique(0xB8 + YLigne1 - 1);	/* incrmente page. */
	}
	Adresse.X = X1;	/* Sauvegarde x1. */
	Adresse.YLigne = YLigne1;	/* Sauvegarde YLigne1 */
}

/**************************************************************************************
void EffacerAfficheurGraphique(unsigned char Positif)

Efface l'cran et positionne le curseur en haut  gauche (1, 1).

Positif = 0 : Ecran  fond blanc ; Positif = 1 : Ecran  fond noir.

**************************************************************************************/
void EffacerAfficheurGraphique(unsigned char Positif)
{
	int i = 0;

	PositionnerCurseurXYGraphique(1, 1);	/* Positionnement du curseur en haut  gauche. */

	while(i < NOMBRE_OCTETS_ECRAN)	/* Efface ou affiche toutes les pages du LCD. */
	{
		if(Positif)
		{
			EnvoyerDonneeGraphique(0xFF);
		}
		else
		{
			EnvoyerDonneeGraphique(0x00);
		}
		i++;
	}
	PositionnerCurseurXYGraphique(1, 1);	/* Positionnement du curseur en haut  gauche. */
}

/**************************************************************************************
void AllumerEteindreAfficheur(unsigned char onoff)

Allume ou teint l'affichage. La RAM n'est pas affecte. 

Choix = 1 : Allum ; Choix = 0 : Eteint.

**************************************************************************************/
void AllumerEteindreAfficheur(unsigned char Choix)
{

	if(Choix == 1)
	{
		SelectionnerBlocGraphique(0);
		EnvoyerControleGraphique(0x3F);	/* Active l'affichage ct gauche. */
		SelectionnerBlocGraphique(1);
		EnvoyerControleGraphique(0x3F);	/* Active l'affichage ct droit. */
	}
	else
	{
		SelectionnerBlocGraphique(0);
		EnvoyerControleGraphique(0x3E);	/* Dsactive l'affichage ct gauche. */
		SelectionnerBlocGraphique(1);
		EnvoyerControleGraphique(0x3E);	/* Dsactive l'affichage du ct gauche. */
	}
}

/**************************************************************************************
void aff_image(unsigned char *Donnee, unsigned char Positif)

Affiche (Donnee) une image complte (128 X 64 pixels) ; Positif = 1 : A fond blanc ; Positif = 0 : A fond  noir.

**************************************************************************************/
void AfficherImageGraphique(const unsigned char *Image, unsigned char Positif)
{
	int k = 0;

	PositionnerCurseurXYGraphique(1, 1);

	while(k < NOMBRE_OCTETS_ECRAN)	/* Ecriture de  NOMBRE_OCTETS_ECRAN  octets constituant l'image. */
	{
		if(Positif == 1)
		{
			EnvoyerDonneeGraphique(*Image);
		}
		else
		{
			EnvoyerDonneeGraphique(~*Image);
		}
		Image++;
		k++;
	}
}



/* Afficher un caractre, selon le choix des atributs voulus.
Attributs :
(ATTRIBUT_ATTENTE_1 | ATTRIBUT_ATTENTE_2 | ATTRIBUT_HAUTEUR_DOUBLE | ATTRIBUT_LARGEUR_DOUBLE | ATTRIBUT_ATTENTE_3 | ATTRIBUT_BARRE | ATTRIBUT_SOULIGNE | ATTRIBUT_POSITIF).
*/

void AfficherCaractereGraphique(char Caractere, unsigned char Attribut)
{
	unsigned char k;
	unsigned char v;

	int DebutX;
	int DebutY;

	unsigned char Tampon1 = 0;
	unsigned char Tampon2 = 0;
	unsigned char TamponSouligneBarre = 0x00;
	unsigned char TamponSouligneBarre2 = 0x00;
	unsigned char TamponSouligneBarre3 = 0x00;

	unsigned char IterationsHauteur = 1;
	unsigned char IterationsLargeur = 1;

	DebutX = Adresse.X;
	DebutY = Adresse.YLigne;

	if(((Attribut & 0x04) == ATTRIBUT_BARRE))	/* On souhaite que le caractre soir affich de faon barre. */
	{
		TamponSouligneBarre = 0x08;
		TamponSouligneBarre3 = 0xC0;
	}

	if(((Attribut & 0x02) == ATTRIBUT_SOULIGNE))	/* On souhaite que le caractre soir affich de faon souligne. */
	{
		TamponSouligneBarre |= 0x80;
		TamponSouligneBarre2 |= 0xC0;
	}

	if(((Attribut & 0x10) == ATTRIBUT_LARGEUR_DOUBLE))	/* On souhaite que le caractre soir affich en taille 10 par 7 pixels. */
	{
		IterationsLargeur = 2;
	}

	if(((Attribut & 0x20) == ATTRIBUT_HAUTEUR_DOUBLE))	/* On souhaite que le caractre soir affich en taille 5 par 14 pixels. */
	{
		IterationsHauteur = 1;
	}

	if(((Attribut & 0x30) == (ATTRIBUT_HAUTEUR_DOUBLE | ATTRIBUT_LARGEUR_DOUBLE)))	/* On souhaite que le caractre soir affich en taille 10 par 14 pixels. */
	{
		IterationsHauteur = 2;
	}

	if(((Attribut & 0x30) != (ATTRIBUT_HAUTEUR_DOUBLE | ATTRIBUT_LARGEUR_DOUBLE)))	/* On souhaite que le caractre soir affich en taille 5 par 7 pixels. */
	{
		if((128 - DebutX) < 5)
		{
			PositionnerCurseurXYGraphique(1, (DebutY + 1));
		}
	}
	else
	{
		if(((Attribut & 0x20) == ATTRIBUT_HAUTEUR_DOUBLE))	/* On souhaite que le caractre soir affich en taille 10 par 14 pixels. */
		{
			if((128 - DebutX) < 10)
			{
				PositionnerCurseurXYGraphique(1, (DebutY + 2));
				DebutX = Adresse.X;
				DebutY = Adresse.YLigne;
			}
		}
	}

	if(((Attribut & 0x01) == ATTRIBUT_NORMAL))	/* On souhaite que le caractre soir affich en ngatif : en blanc sur fond noir */
	{
		if(((Attribut & 0x20) != ATTRIBUT_HAUTEUR_DOUBLE))	/* On souhaite que le caractre soir affich en taille 5 par 7 pixels, ou en taille 10 par 7 pixels. */
		{
			for(k = 0; k < 5; k++)
			{
				for(v = 0; v < IterationsLargeur; v++)
				{
					EnvoyerDonneeGraphique((~(TamponSouligneBarre | Font5x7[(((int)Caractere) - 0x20) * 5 + k])));
				}
			}
			for(v = 0; v < IterationsLargeur; v++)	/* Affichage d'une, ou de deux colonnes de pixels vides aprs chaque caractre */
			{
				EnvoyerDonneeGraphique(((~TamponSouligneBarre) & 0xFF));
			}
		}
		else	/* On souhaite que le caractre soir affich en taille 10 par 14 pixels, ou en taille 7 par 14 pixels. */
		{
			for(k = 0; k < 5; k++)	/* Affichage de la moiti suprieure du caractre */
			{
				Tampon2 = 0x00;
				Tampon1 = (TamponSouligneBarre | Font5x7[(((int)Caractere) - 0x20) * 5 + k]);
				Tampon2 |= (((Tampon1 & 0x08) << 4) + ((Tampon1 & 0x08) << 3));
				Tampon2 |= (((Tampon1 & 0x04) << 3) + ((Tampon1 & 0x04) << 2));
				Tampon2 |= (((Tampon1 & 0x02) << 2) + ((Tampon1 & 0x02) << 1));
				Tampon2 |= (((Tampon1 & 0x01) << 1) + (Tampon1 & 0x01));

				for(v = 0; v < IterationsHauteur; v++)
				{
					EnvoyerDonneeGraphique((~(TamponSouligneBarre3 | Tampon2)));
				}
			}

			for(v = 0; v < IterationsHauteur; v++)	/* Affichage d'une, ou de deux colonnes de pixels vides aprs chaque caractre */
			{
				EnvoyerDonneeGraphique(((~TamponSouligneBarre3) & 0xFF));
			}

			PositionnerCurseurXYGraphique(DebutX, (DebutY + 1));

			for(k = 0; k < 5; k++)	/* Affichage de la moiti infrieure du caractre */
			{
				Tampon2 = 0x00;
				Tampon1 = (TamponSouligneBarre | Font5x7[(((int)Caractere) - 0x20) * 5 + k]);
				Tampon2 |= ((Tampon1 & 0x80) + ((Tampon1 & 0x80) >> 1));
				Tampon2 |= (((Tampon1 & 0x40) >> 1) + ((Tampon1 & 0x40) >> 2));
				Tampon2 |= (((Tampon1 & 0x20) >> 2) + ((Tampon1 & 0x20) >> 3));
				Tampon2 |= (((Tampon1 & 0x10) >> 3) + ((Tampon1 & 0x10) >> 4));

				for(v = 0; v < IterationsHauteur; v++)
				{
					EnvoyerDonneeGraphique(~Tampon2);
				}
			}

			for(v = 0; v < IterationsHauteur; v++)	/* Affichage d'une, ou de deux colonnes de pixels vides aprs chaque caractre */
			{
				EnvoyerDonneeGraphique((~(TamponSouligneBarre2) & 0xFF));
			}

			PositionnerCurseurXYGraphique((DebutX + (IterationsLargeur * 6)), DebutY);
		}
	}
	else	/* On souhaite que le caractre soir affich en positif : en noir sur fond blanc. */
	{
		if(((Attribut & 0x20) != ATTRIBUT_HAUTEUR_DOUBLE))	/* On souhaite que le caractre soir affich en taille 5 par 7 pixels, ou en taille 10 par 7 pixels. */
		{
			for(k = 0; k < 5; k++)
			{
				for(v = 0; v < IterationsLargeur; v++)
				{
					EnvoyerDonneeGraphique((TamponSouligneBarre | Font5x7[(((int)Caractere) - 0x20) * 5 + k]));
				}
			}
			for(v = 0; v < IterationsLargeur; v++)	/* Affichage d'une, ou de deux colonnes de pixels vides aprs chaque caractre */
			{
				EnvoyerDonneeGraphique((TamponSouligneBarre));
			}
		}
		else	/* On souhaite que le caractre soir affich en taille 10 par 14 pixels, ou en taille 7 par 14 pixels. */
		{
			for(k = 0; k < 5; k++)	/* Affichage de la moiti suprieure du caractre */
			{
				Tampon2 = 0x00;
				Tampon1 = (TamponSouligneBarre | Font5x7[(((int)Caractere) - 0x20) * 5 + k]);
				Tampon2 |= ((Tampon1 & 0x08) << 4) + ((Tampon1 & 0x08) << 3);
				Tampon2 |= ((Tampon1 & 0x04) << 3) + ((Tampon1 & 0x04) << 2);
				Tampon2 |= ((Tampon1 & 0x02) << 2) + ((Tampon1 & 0x02) << 1);
				Tampon2 |= ((Tampon1 & 0x01) << 1) + (Tampon1 & 0x01);

				for(v = 0; v < IterationsHauteur; v++)
				{
					EnvoyerDonneeGraphique(Tampon2);
				}
			}

			for(v = 0; v < IterationsHauteur; v++)	/* Affichage d'une, ou de deux colonnes de pixels vides aprs chaque caractre */
			{
				EnvoyerDonneeGraphique((TamponSouligneBarre3));
			}

			PositionnerCurseurXYGraphique(DebutX, (DebutY + 1));

			for(k = 0; k < 5; k++)	/* Affichage de la moiti infrieure du caractre */
			{
				Tampon2 = 0x00;
				Tampon1 = (TamponSouligneBarre | Font5x7[(((int)Caractere) - 0x20) * 5 + k]);
				Tampon2 |= (Tampon1 & 0x80) + ((Tampon1 & 0x80)	>> 1);
				Tampon2 |= ((Tampon1 & 0x40) >> 1) + ((Tampon1 & 0x40) >> 2);
				Tampon2 |= ((Tampon1 & 0x20) >> 2) + ((Tampon1 & 0x20) >> 3);
				Tampon2 |= ((Tampon1 & 0x10) >> 3) + ((Tampon1 & 0x10) >> 4);

				for(v = 0; v < IterationsHauteur; v++)
				{
					EnvoyerDonneeGraphique(Tampon2);
				}
			}

			for(v = 0; v < IterationsHauteur; v++)
			{
				EnvoyerDonneeGraphique((TamponSouligneBarre2));	/* Affichage d'une ou de deux colonnes de pixels vides aprs chaque caractre */
			}

			PositionnerCurseurXYGraphique((DebutX + (IterationsLargeur * 6)), DebutY);
		}
	}
}



void AfficherTexteGraphique(const rom char * Texte, unsigned char Attribut)
{
	int k = 0;

	while(Texte[k] != '\0')
	{
		if(Texte[k] == '\r')	/* Il y a un aller  la ligne suivante de souhait. */
		{
			if(((Attribut & 0x20) != ATTRIBUT_HAUTEUR_DOUBLE))	/* On souhaite que le caractre soir affich en taille 5 par 7 pixels, ou en taille 10 par 7 pixels. */
			{
				PositionnerCurseurXYGraphique(1, (Adresse.YLigne + 1));	/* On positionne le curseur une ligne suivant la ligne utilise. */
			}
			else	/* On souhaite que le caractre soir affich en taille 10 par 14 pixels, ou en taille 7 par 14 pixels. */
			{
				PositionnerCurseurXYGraphique(1, (Adresse.YLigne + 2));	/* On positionne le curseur deux lignes aprs la ligne utilise. */
			}
		}
	AfficherCaractereGraphique(Texte[k], Attribut);
	k++;
	}
}

void ConvertirAfficherGraphique(int Nombre, unsigned char Base, unsigned char Attribut)
{
	int k = 0;     		        /* Compteur. */
	unsigned char Tampon[32];

	if(Nombre < 0)	/* Si le nombre  convertir et  afficher est strictement ngatif */
	{
		Nombre = (-Nombre);
		AfficherCaractereGraphique('-', Attribut);	/* Affichage du signe  Moins  */
	}

	/* Remplissage de  Tampon  par le code ASCII du rsultat de l'opration (Nombre % Base),
	grce  la constante  Hexadecimaux  dPositifare en en-tte de ce fichier */
	do
	{
		Tampon[k] = Hexadecimaux[Nombre % Base];    /* Conversion Hexadcimale / ASCII de la fin de la chane de caractres vers le dbut. */
		Nombre = (Nombre / Base);
		k++;
	}
	while(Nombre > 0);   			

	/* Affichage le contenu de  Tampon , de la fin vers le dbut et caractre par caractre, 
	directement dans la RAM du LCD. */
	do
	{
		k--;
		AfficherCaractereGraphique(Tampon[k], Attribut);		
	}
	while(k > 0);
}

unsigned char AfficherLesBlocs(void)
{
	int n = 0;
	int m = 0;
	for(n = 0; n < 8; n++)
	{
		PositionnerCurseurXYGraphique(1, (n + 1));
		for(m = 0; m < 64; m++)
		{
		 	EnvoyerDonneeGraphique(0xFF);
		}
		PositionnerCurseurXYGraphique(7, 3);
		AfficherTexteGraphique("Bloc 0\0", ATTRIBUT_NORMAL);
	}
	for(n = 0; n < 8; n++)
	{
		PositionnerCurseurXYGraphique(65, (n + 1));
		for(m = 0; m < 64; m++)
		{
		 	EnvoyerDonneeGraphique(0x00);
		}
		PositionnerCurseurXYGraphique(70, 3);
		AfficherTexteGraphique("Bloc 1\0", ATTRIBUT_POSITIF);
	}
	return 1;
}

unsigned char AfficherLesPages(void)
{
	int n = 0;
	int m = 0;
	unsigned char Alt = 0;
	Alt = 0xFF;
	for(n = 0; n < 8; n++)
	{
		PositionnerCurseurXYGraphique(1, (n + 1));
		for(m = 0; m < 64; m++)
		{
		 	EnvoyerDonneeGraphique(Alt);
		}
		PositionnerCurseurXYGraphique(7, (n + 1));
		if(Alt == 0x00)
		{
			AfficherTexteGraphique("Page 0-\0", ATTRIBUT_POSITIF);
			ConvertirAfficherGraphique(n, 10, ATTRIBUT_POSITIF);
			Alt = 0xFF;
		}
		else
		{
			AfficherTexteGraphique("Page 0-\0", ATTRIBUT_NORMAL);
			ConvertirAfficherGraphique(n, 10, ATTRIBUT_NORMAL);
			Alt = 0x00;
		}
		PositionnerCurseurXYGraphique(65, (n + 1));
		for(m = 0; m < 64; m++)
		{
		 	EnvoyerDonneeGraphique(Alt);
		}
		PositionnerCurseurXYGraphique(70, (n + 1));
		if(Alt == 0x00)
		{
			AfficherTexteGraphique("Page 1-\0", ATTRIBUT_POSITIF);
			ConvertirAfficherGraphique(n, 10, ATTRIBUT_POSITIF);
		}
		else
		{
			AfficherTexteGraphique("Page 1-\0", ATTRIBUT_NORMAL);
			ConvertirAfficherGraphique(n, 10, ATTRIBUT_NORMAL);
		}
	}
	return 1;
}

/**************************************************************************************
void RemplirZoneGraphique(unsigned char Largeur, unsigned char Hauteur, unsigned char Donnee)

 Remplit une zone,  partir de la position du curseur, par l'octet Donnee. Hauteur (Hauteur) 
 doit tre multiple de 8.

Paramtres

-	Largeur : Largeur,
-	Hauteur : Hauteur,

**************************************************************************************/
void RemplirZoneGraphique(unsigned char Largeur, unsigned char Hauteur, unsigned char Donnee)
{

	unsigned char j = 0x00, k = 0x00;

	while (k < (Hauteur / 8))	/* Remplit la zone page par page. */ 
	{

		for (j = 0; j < Largeur; j++)
		{
			EnvoyerDonneeGraphique(Donnee);
		}

		k++;
		Adresse.YLigne++;
		Adresse.X = Adresse.X - Largeur;
		PositionnerCurseurXYGraphique(Adresse.X, Adresse.YLigne);
	};
}

/**************************************************************************************
void TracerLogoGraphique(unsigned char Largeur, unsigned char Hauteur, const unsigned char *Donnee, unsigned char Positif)

Trace,  partir de la position du curseur, un logo Donnee qui doit avoir pour Hauteur un 
multiple de 8  pixels.

Paramtres

-	Largeur : Largeur,
-	Hauteur : Hauteur,
-	*Donnee : logo,
-	Positif = 1 : A fond blanc ; Positif = 0 : A fond noir.

**************************************************************************************/
void TracerLogoGraphique(unsigned char Largeur, unsigned char Hauteur, const unsigned char *Donnee, unsigned char Positif) {

	unsigned char i = 0, j = 0;

	while (j < (Hauteur/8)){

		for (i = 0; i < Largeur; i++) {

			if (Positif == 1)
				EnvoyerDonneeGraphique(*Donnee);	/* Envoie  Donnee . */
			else
				EnvoyerDonneeGraphique(~*Donnee);	/* Envoie le complment de  Donnee . */

			Donnee++ ;
		};

		j++;
		Adresse.YLigne++;
		Adresse.X = Adresse.X - Largeur;
	};
}

/**************************************************************************************
void TracerPointXYGraphique(unsigned char X, unsigned char Y, unsigned char Positif)

Trace un point de coordonnes (X, Y) tel que 1 <= X <= 128 et 1 <= Y <= 64.

Positif = 1 : Point noir ; Positif = 0 : Point blanc.

**************************************************************************************/
void TracerPointXYGraphique(unsigned char X, unsigned char Y, unsigned char Positif)
{
	unsigned char PointXY;
	unsigned char TamponY;

	TamponY	= (((Y - 1) / 8) + 1);

	PositionnerCurseurXYGraphique(X, TamponY);

	PointXY = LireDonneeGraphique(); 			/* Transfert de l'octet lu vers  PointXY . */

	PositionnerCurseurXYGraphique(X, TamponY);

	if(Positif)
	{
		PointXY |= (1 << (Y - 1) % 8);
	}
	else
	{
		PointXY &= (~(1 << (Y - 1) % 8));
	}

	EnvoyerDonneeGraphique(PointXY);
}

/**************************************************************************************
void TracerCadreGraphique(unsigned char x, unsigned char y, unsigned char L, unsigned char H, unsigned char Positif)

Trace ou efface un cadre.

Paramtres

-  (x, y) : coordonnes du point Hauteur gauche du cadre,
-  Largeur : Largeur,
-  Hauteur : Hauteur,
-  Positif = 1 : Cadre noir; Positif = 0 : Cadre blanc.

**************************************************************************************/
void TracerCadreGraphique(unsigned char X, unsigned char Y, unsigned char Largeur, unsigned char Hauteur, unsigned char Positif)
{

	unsigned char j;

	for (j = 0; j < Hauteur; j++)
	{
		TracerPointXYGraphique(X, Y + j, Positif);	/* Traage du ct vertical gauche. */
		TracerPointXYGraphique(X + Largeur - 1, Y + j, Positif);	/* Traage du ct vertical droit. */
	}

	for (j = 0; j < Largeur; j++)
	{
		TracerPointXYGraphique(X + j, Y, Positif);	/* Traage du ct horizontal haut. */
		TracerPointXYGraphique(X + j, Y + Hauteur - 1, Positif);	/* Traage du cte horizontal bas. */
	}
}

/**************************************************************************************
void TracerRectangleGraphique(unsigned char X, unsigned char Y, unsigned char Largeur, unsigned char Hauteur, unsigned char Positif)

Trace ou efface un rectangle plein.

Paramtres

-  (x, y) : coordonnes du point Hauteur gauche du rectangle,
-  Largeur : Largeur,
-  Hauteur : Hauteur,
-  Positif=1 : affiche; Positif=0 : efface.
 
**************************************************************************************/
void TracerRectangleGraphique(unsigned char X, unsigned char Y, unsigned char Largeur, unsigned char Hauteur, unsigned char Positif)
{
	unsigned char i, j;

	for (i=0;i< Hauteur;i++)
	{
		for (j = 0; j < Largeur; j++)
		{
			TracerPointXYGraphique(X+i, Y+j,Positif);
		}
	}
}

/**************************************************************************************
void Tempo_ms(int Temps)

Temporisation logicielle approximative avec un pas de #1ms.
 
**************************************************************************************/
void Tempo_ms(int Temps)
{

	int j = 0, i = 0;

	for (j = 0; j < Temps; j++)			/* Rpte temps x 1ms. */
	{
		for (i = 0; i < 90; i++); 		/* Un pas de #1ms pour un quartz de #10MHz. */
	}
}


